T_SUB "__zplug::core::core::get_interfaces" ((
  # skip
))
T_SUB "__zplug::core::core::run_interfaces" ((
  # skip
))
T_SUB "__zplug::core::core::prepare" ((
  # skip
))
T_SUB "__zplug::core::core::variable" ((
  # skip
))
