T_SUB "__zplug::core::options::get" ((
  # skip
))
T_SUB "__zplug::core::options::parse" ((
  # skip
))
T_SUB "__zplug::core::options::short" ((
  # skip
))
T_SUB "__zplug::core::options::long" ((
  # skip
))
T_SUB "__zplug::core::options::unknown" ((
  # skip
))
