T_SUB "__zplug::sources::gist::check" ((
  # skip
))
T_SUB "__zplug::sources::gist::install" ((
  # skip
))
T_SUB "__zplug::sources::gist::update" ((
  # skip
))
T_SUB "__zplug::sources::gist::get_url" ((
  # skip
))
T_SUB "__zplug::sources::gist::load_plugin" ((
  # skip
))
T_SUB "__zplug::sources::gist::load_command" ((
  # skip
))
