T_SUB "__zplug::base::base::is_cli" ((
  # skip
))
T_SUB "__zplug::base::base::zpluged" ((
  # skip
))
T_SUB "__zplug::base::base::version_requirement" ((
  # skip
))
T_SUB "__zplug::base::base::git_version" ((
  # skip
))
T_SUB "__zplug::base::base::zsh_version" ((
  # skip
))
T_SUB "__zplug::base::base::osx_version" ((
  # skip
))
T_SUB "__zplug::base::base::get_os" ((
  # skip
))
T_SUB "__zplug::base::base::is_osx" ((
  # skip
))
T_SUB "__zplug::base::base::is_linux" ((
  # skip
))
T_SUB "__zplug::base::base::packaging" ((
  # skip
))
