T_SUB "__zplug::sources::github::check" ((
  # skip
))
T_SUB "__zplug::sources::github::install" ((
  # skip
))
T_SUB "__zplug::sources::github::update" ((
  # skip
))
T_SUB "__zplug::sources::github::get_url" ((
  # skip
))
T_SUB "__zplug::sources::github::load_plugin" ((
  # skip
))
T_SUB "__zplug::sources::github::load_command" ((
  # skip
))
