T_SUB "__zplug::job::queue::enqueue" ((
  # skip
))
T_SUB "__zplug::job::queue::dequeue" ((
  # skip
))
T_SUB "__zplug::job::queue::clear" ((
  # skip
))
T_SUB "__zplug::job::queue::wait" ((
  # skip
))
T_SUB "__zplug::job::queue::wait_all" ((
  # skip
))
