T_SUB "__zplug::utils::shell::remove_deadlinks" ((
  # skip
))
T_SUB "__zplug::utils::shell::search_commands" ((
  # skip
))
T_SUB "__zplug::utils::shell::glob2regexp" ((
  # skip
))
T_SUB "__zplug::utils::shell::sudo" ((
  # skip
))
T_SUB "__zplug::utils::shell::unansi" ((
  # skip
))
T_SUB "__zplug::utils::shell::cd" ((
  # skip
))
T_SUB "__zplug::utils::shell::getopts" ((
  # skip
))
T_SUB "__zplug::utils::shell::pipestatus" ((
  # skip
))
