T_SUB "__zplug::core::sources::is_exists" ((
  # skip
))
T_SUB "__zplug::core::sources::is_handler_defined" ((
  # skip
))
T_SUB "__zplug::core::sources::use_handler" ((
  # skip
))
T_SUB "__zplug::core::sources::call" ((
  # skip
))
T_SUB "__zplug::core::sources::use_default" ((
  # skip
))
