T_SUB "__zplug::sources::gh-r::check" ((
  # skip
))
T_SUB "__zplug::sources::gh-r::install" ((
  # skip
))
T_SUB "__zplug::sources::gh-r::update" ((
  # skip
))
T_SUB "__zplug::sources::gh-r::load_command" ((
  # skip
))
