T_SUB "__zplug::utils::git::clone" ((
  # skip
))
T_SUB "__zplug::utils::git::checkout" ((
  # skip
))
T_SUB "__zplug::utils::git::merge" ((
  # skip
))
T_SUB "__zplug::utils::git::status" ((
  # skip
))
T_SUB "__zplug::utils::git::get_head_branch_name" ((
  # skip
))
T_SUB "__zplug::utils::git::get_remote_name" ((
  # skip
))
T_SUB "__zplug::utils::git::get_remote_state" ((
  # skip
))
T_SUB "__zplug::utils::git::get_state" ((
  # skip
))
T_SUB "__zplug::utils::git::remote_url" ((
  # skip
))
