T_SUB "__zplug::sources::oh-my-zsh::check" ((
  # skip
))
T_SUB "__zplug::sources::oh-my-zsh::install" ((
  # skip
))
T_SUB "__zplug::sources::oh-my-zsh::update" ((
  # skip
))
T_SUB "__zplug::sources::oh-my-zsh::get_url" ((
  # skip
))
T_SUB "__zplug::sources::oh-my-zsh::load_plugin" ((
  # skip
))
