T_SUB "__zplug::utils::releases::get_latest" ((
  # skip
))
T_SUB "__zplug::utils::releases::get_state" ((
  # skip
))
T_SUB "__zplug::utils::releases::is_64" ((
  # skip
))
T_SUB "__zplug::utils::releases::get_url" ((
  # skip
))
T_SUB "__zplug::utils::releases::get" ((
  # skip
))
T_SUB "__zplug::utils::releases::index" ((
  # skip
))
