T_SUB "__zplug::sources::bitbucket::check" ((
  # skip
))
T_SUB "__zplug::sources::bitbucket::install" ((
  # skip
))
T_SUB "__zplug::sources::bitbucket::update" ((
  # skip
))
T_SUB "__zplug::sources::bitbucket::get_url" ((
  # skip
))
T_SUB "__zplug::sources::bitbucket::load_plugin" ((
  # skip
))
T_SUB "__zplug::sources::bitbucket::load_command" ((
  # skip
))
